from rest_framework import generics


class UpdateAPIView(generics.UpdateAPIView):
    """ Base UpdateAPIView with following additions:
        - POST request method was added
    """

    http_method_names = ['post', 'patch', ]

    def post(self, request, *args, **kwargs):
        return self.patch(request, *args, **kwargs)

    def perform_update(self, serializer):
        # log_serializer_change(self.request, self.get_object(), serializer=serializer)
        serializer.save()


class DestroyAPIView(generics.DestroyAPIView):
    """ Base DestroyAPIView with following additions:
        - POST request method added;
        - does not perform actual database deletion, only marks object as deleted
    """
    http_method_names = ['post', 'delete', ]

    def post(self, request, *args, **kwargs):
        return self.destroy(request, *args, **kwargs)

    def perform_destroy(self, instance):
        instance.is_deleted = True
        instance.save()



from django.utils.functional import Promise
from django.utils.encoding import force_text
from django.core.serializers.json import DjangoJSONEncoder


class LazyEncoder(DjangoJSONEncoder):
    def default(self, obj):
        if isinstance(obj, Promise):
            return force_text(obj)
        return super(LazyEncoder, self).default(obj)




import ast

from rest_framework import serializers


class ChoiceField(serializers.ChoiceField):

    def to_representation(self, obj):
        if obj:
            return {'value': obj, 'display_name': self._choices[obj]}
        return obj


class MultipleChoiceField(serializers.MultipleChoiceField):

    def to_internal_value(self, data):
        if isinstance(data, str) or not hasattr(data, '__iter__'):
            self.fail('not_a_list', input_type=type(data).__name__)
        if not self.allow_empty and len(data) == 0:
            self.fail('empty')

        return list(
            item for item in data
        )

    def to_representation(self, value):
        try:
            value = ast.literal_eval(str(value))
        except (ValueError, SyntaxError):
            value = list(value, )
        result = list(
            {
                'value': item,
                'display_name': self.get_display_name(item)
            } for item in value
        )
        return result

    def get_display_name(self, item):
        choices = self.choices
        try:
            return next(choices[choice] for choice in choices if choice == item)
        except StopIteration:
            return item

